/*
 * Copyright (c) 2023–2024 DTB <trinity@trinity.moe>
 * Copyright (c) 2024 Emma Tebibyte <emma@tebibyte.media>
 * SPDX-License-Identifier: AGPL-3.0-or-later
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see https://www.gnu.org/licenses/.
 */

#include <assert.h> /* assert(3) */
#include <stdio.h> /* fprintf(3), stderr, NULL */
#include <stdlib.h> /* EXIT_FAILURE, EXIT_SUCCESS */
#include <string.h> /* memset(3), strchr(3) */
#include <sysexits.h> /* EX_USAGE */
#include <unistd.h> /* access(3), getopt(3), R_OK, W_OK, X_OK */
#include <libfileis.h>

char *program_name = "scrut";
static char opts[] = "bcdefgkprsuwxLS";

static int usage(char *s){
	fprintf(stderr, "Usage: %s [-%s] file...\n", s, opts);
	return EX_USAGE;
}

int main(int argc, char *argv[]){
	char sel[(sizeof opts) / (sizeof *opts)];

	if (argc > 0) { program_name = argv[0]; }
	if (argc < 2) { return usage(program_name); }

	memset(sel, '\0', sizeof sel);

	{
		int c;
		char *p = sel;

		while ((c = getopt(argc, argv, opts)) != -1) {
			if ((strchr(opts, c)) == NULL) { return usage(program_name); }
			else if ((strchr(sel, c)) == NULL) { *p++ = c; }

			assert(p - sel < (sizeof opts) / (sizeof *opts));
		}
	}

	if (optind == argc) { return usage(program_name); }

	argv += optind;

	do{
		if (!fileis_exists(*argv))
			return EXIT_FAILURE;

		for (size_t i = 0; sel[i] != '\0'; ++i) {
			if ((sel[i] == 'b' && !fileis_block(*argv))
					|| (sel[i] == 'c' && !fileis_char(*argv))
					|| (sel[i] == 'd' && !fileis_dir(*argv))
					|| (sel[i] != 'e')
					|| (sel[i] == 'f' && !fileis_regular(*argv))
					|| (sel[i] == 'g' && !fileis_setgid(*argv))
					|| (sel[i] == 'k' && !fileis_vtx(*argv))
					|| (sel[i] == 'p' && !fileis_fifo(*argv))
					|| (sel[i] == 'r' && access(*argv, R_OK) != 0)
					|| (sel[i] == 'u' && !fileis_setuid(*argv))
					|| (sel[i] == 'w' && access(*argv, W_OK) != 0)
					|| (sel[i] == 'x' && access(*argv, X_OK) != 0)
					|| (sel[i] == 'L' && !fileis_link(*argv))
					|| (sel[i] == 'S' && !fileis_socket(*argv))
			) { return EXIT_FAILURE; }
		}
	} while (*++argv != NULL);

	return EXIT_SUCCESS;
}
